<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    INVERTUS, UAB www.invertus.eu <support@invertus.eu>
 * @copyright Copyright (c) permanent, INVERTUS, UAB
 * @license   Addons PrestaShop license limitation
 * @see       /LICENSE
 *
 * International Registered Trademark & Property of INVERTUS, UAB
 */

namespace Invertus\SwedBank\Controller;

use Invertus\SwedBank\Adapter\Configuration;
use Invertus\SwedBank\Service\PaymentMethodProvider;
use ModuleAdminController;
use OrderState;
use Collection;
use SwedBank;
use Invertus\SwedBank\Config\Config;
use Tab;
use Tools;
use Validate;

abstract class AbstractAdminController extends ModuleAdminController
{
    const FILENAME = 'AbstractAdminController';

    /**
     * @var  SwedBank
     */
    public $module;

    public $bootstrap = true;

    /**
     * @var  Configuration
     */
    public $configuration;

    public function __construct()
    {
        parent::__construct();
        $this->configuration = $this->get(Configuration::class);
    }

    /**
     * Quick access to service container
     *
     * @param string $id
     *
     * @return mixed
     */
    public function get($id)
    {
        return $this->module->get($id);
    }

    public function initContent()
    {
        if ($this->ajax) {
            return parent::initContent();
        }

        $this->content .= $this->renderNavigationTop();

        parent::initContent();

        $this->content .= $this->renderNavigationBottom();
        $this->context->smarty->assign('content', $this->content);
    }

    protected function getAdminTemplatePath()
    {
        return $this->module->getLocalPath().'views/templates/admin/';
    }

    protected function getAdminJsUrl()
    {
        return $this->module->getPathUri().'views/js/admin/';
    }

    protected function getAdminCssUrl()
    {
        return $this->module->getPathUri().'views/css/admin/';
    }

    /**
     * gets stored configuration name of the bank. Otherwise will return 0
     * @return int|string
     */
    protected function getStoredBankConfigurationName()
    {
        $banks = $this->configuration->getMultiple([Config::BANK_LT, Config::BANK_LV, Config::BANK_EE]);

        foreach ($banks as $configName => $status) {
            if ($status) {
                return $configName;
            }
        }

        return 0;
    }

    /**
     * checks if form was disabled or not
     * @return int
     */
    protected function isFormDisabled()
    {
        $isFormDisabled = 0;

        switch ($this->controller_name) {
            case SwedBank::ADMIN_TEST_GENERAL_SETTINGS_CONTROLLER:
                $isFormDisabled = (int) $this->configuration->get(Config::LIVE_ENVIRONMENT) ||
                    !$this->configuration->get(Config::TERMS_AND_CONDITIONS_TEST);
                break;
            case SwedBank::ADMIN_LIVE_GENERAL_SETTINGS_CONTROLLER:
                $isFormDisabled = (int) $this->configuration->get(Config::TEST_ENVIRONMENT) ||
                    !$this->configuration->get(Config::TERMS_AND_CONDITIONS_LIVE);
                break;
        }
        return $isFormDisabled;
    }

    /**
     * formats array with all field sets
     * @return array - arrayt which is set for Prestashops fields_form
     */
    protected function getLiveTestingFormFields()
    {
        $environment = '';
        $environmentLabelName = '';
        $disableStatus = 0;

        switch ($this->controller_name) {
            case SwedBank::ADMIN_TEST_GENERAL_SETTINGS_CONTROLLER:
                $environment = Config::TEST_ENVIRONMENT;
                $environmentLabelName = $this->module->l('Enable test environment', self::FILENAME);
                $disableStatus = $this->configuration->get(Config::LIVE_ENVIRONMENT) ||
                    !$this->configuration->get(Config::TERMS_AND_CONDITIONS_TEST);
                break;
            case SwedBank::ADMIN_LIVE_GENERAL_SETTINGS_CONTROLLER:
                $environment = Config::LIVE_ENVIRONMENT;
                $environmentLabelName = $this->module->l('Enable live environment', self::FILENAME);
                $disableStatus = $this->configuration->get(Config::TEST_ENVIRONMENT) ||
                    !$this->configuration->get(Config::TERMS_AND_CONDITIONS_LIVE);
                break;
        }

        $countryBlockEnabled = $this->getStoredBankConfigurationName();

        if (!$countryBlockEnabled) {
            return [
                $this->getTermsAndConditionsFormFields($environment, $environmentLabelName, $disableStatus),
                $this->getCountryFormFields($environment, $disableStatus),
            ];
        }

        return [
            $this->getTermsAndConditionsFormFields($environment, $environmentLabelName, $disableStatus),
            $this->getCountryFormFields($environment, $disableStatus),
            $this->getOrderStatusFormFields($environment, $disableStatus)
        ];
    }

    /**
     * sets fields values for current form fields
     */
    protected function setLiveTestFormValues()
    {
        $this->fields_value[Config::LANGUAGE_TYPE_CARD.'_text'] =
            $this->module->l('Page set ID for card payment', self::FILENAME);
        $this->fields_value['banklink_text'] =
            $this->module->l('Banklink payment (language on bank page)', self::FILENAME);

        if (!empty($this->errors)) {
            // notify javascript that form has to be enabled on
            $this->fields_value['hasError'] = 1;
            $this->fields_value['disable_form'] = 0;
            $this->refillFieldValues();
            return;
        }
        $countryBlockEnabled = $this->getStoredBankConfigurationName();
        $environment = $this->getEnvironment();

        switch ($environment) {
            default:
            case Config::LIVE_ENVIRONMENT:
                $env = SwedBank::ENV_LIVE;
                $tosInputName = Config::TERMS_AND_CONDITIONS_LIVE;
                $clientIdInputName = Config::CLIENT_ID_LIVE;
                $clientPasswordInputName = Config::CLIENT_PASSWORD_LIVE;
                $paymentMethodsInputName = Config::PAYMENT_METHODS_LIVE;
                $orderStatusSuccessInputName = Config::ORDER_STATUS_SUCCESS_LIVE;
                $orderStatusAwaitingInputName = Config::ORDER_STATUS_AWAITING_LIVE;
                $debugModeInputName = Config::DEBUG_MODE_LIVE;
                $orderStatusRequiresInvName = Config::ORDER_STATUS_REQUIRES_INVESTIGATION_LIVE;
                break;
            case Config::TEST_ENVIRONMENT:
                $env = SwedBank::ENV_TEST;
                $tosInputName = Config::TERMS_AND_CONDITIONS_TEST;
                $clientIdInputName = Config::CLIENT_ID_TEST;
                $clientPasswordInputName = Config::CLIENT_PASSWORD_TEST;
                $paymentMethodsInputName = Config::PAYMENT_METHODS_TEST;
                $orderStatusSuccessInputName = Config::ORDER_STATUS_SUCCESS_TEST;
                $orderStatusAwaitingInputName = Config::ORDER_STATUS_AWAITING_TEST;
                $debugModeInputName = Config::DEBUG_MODE_TEST;
                $orderStatusRequiresInvName = Config::ORDER_STATUS_REQUIRES_INVESTIGATION_TEST;
                break;
        }

        $this->fields_value['paypal_payment_method_name'] = $paymentMethodsInputName.'_'.Config::PAYMENT_METHOD_PAYPAL;
        $this->fields_value['card_payment_method_name'] = $paymentMethodsInputName.'_'.Config::PAYMENT_METHOD_CARD;


        if (!$countryBlockEnabled) {
            $this->fields_value['no_country_block'] =
            $this->context->smarty->fetch(
                $this->getAdminTemplatePath().'no-country-message.tpl',
                ['settingsLink' => $this->context->link->getAdminLink(SwedBank::ADMIN_CONFIGURATION_CONTROLLER)]
            );
        }
        $this->fields_value[$tosInputName.'_1'] = $this->configuration->get($tosInputName);
        $this->fields_value[Config::TEST_ENVIRONMENT] = $this->configuration->get(Config::TEST_ENVIRONMENT);
        $this->fields_value[Config::LIVE_ENVIRONMENT] = $this->configuration->get(Config::LIVE_ENVIRONMENT);
        $this->fields_value[$clientIdInputName] = $this->configuration->get($clientIdInputName);
        $this->fields_value[$clientPasswordInputName] = $this->configuration->get($clientPasswordInputName);
        $paymentMethods = (array) Tools::jsonDecode($this->configuration->get($paymentMethodsInputName));

        if (!empty($paymentMethods)) {
            foreach ($paymentMethods as $methodId) {
                $this->fields_value[$paymentMethodsInputName.'_'.(int) $methodId] = 1;
            }
        }

        $this->fields_value[$orderStatusSuccessInputName] = $this->configuration->get($orderStatusSuccessInputName);
        $this->fields_value[$orderStatusAwaitingInputName] = $this->configuration->get($orderStatusAwaitingInputName);
        $this->fields_value[$orderStatusRequiresInvName] = $this->configuration->get($orderStatusRequiresInvName);
        $this->fields_value[$debugModeInputName] = $this->configuration->get($debugModeInputName);
        $this->fields_value['disable_form'] = $this->isFormDisabled();

        $languages = \Language::getLanguages(true, $this->context->shop->id);
        foreach ($languages as $language) {
            $swedbankLanguage = new \SwedbankLanguage();
            $result = $swedbankLanguage->getAll([
                'id_language' => $language['id_lang'],
                'id_shop' => $this->context->shop->id,
                'environment' => $env
            ]);

            if (!empty($result)) {
                foreach ($result as $value) {
                    $this->fields_value[$value['type'].'_'.$value['id_language']] = $value['selected_language'];
                }
                continue;
            }

            $bankLtEnabled = (bool) \Configuration::get('SB_BANK_LT');
            $bankLvEnabled = (bool) \Configuration::get('SB_BANK_LV');
            $bankEeEnabled = (bool) \Configuration::get('SB_BANK_EE');

            if ($bankEeEnabled) {
                $configLanguages = Config::getFieldsFormLanguages('EE');
            } elseif ($bankLtEnabled) {
                $configLanguages = Config::getFieldsFormLanguages('LT');
            } elseif ($bankLvEnabled) {
                $configLanguages = Config::getFieldsFormLanguages('LV');
            } else {
                $configLanguages = Config::getFieldsFormLanguages();
            }

            if (in_array(Tools::strtoupper($language['iso_code']), array_column($configLanguages, 'id'))) {
                $this->fields_value['banklink_'.$language['id_lang']] = Tools::strtoupper($language['iso_code']);
            }
        }
    }

    /**
     * submits and then redirects whole form
     */
    protected function submitTestLiveSettings()
    {
        $environment = $this->getEnvironment();

        switch ($environment) {
            default:
            case Config::TEST_ENVIRONMENT:
                $env = SwedBank::ENV_TEST;
                $this->configuration->set(Config::TEST_ENVIRONMENT, 1);
                $this->configuration->set(Config::LIVE_ENVIRONMENT, 0);
                $tosField = Config::TERMS_AND_CONDITIONS_TEST;
                $clientIdField = Config::CLIENT_ID_TEST;
                $clientPasswordField = Config::CLIENT_PASSWORD_TEST;
                $paymentMethodsField = Config::PAYMENT_METHODS_TEST;
                $orderStatusSuccessField = Config::ORDER_STATUS_SUCCESS_TEST;
                $debugModeField = Config::DEBUG_MODE_TEST;
                $orderStatusAwaitingField = Config::ORDER_STATUS_AWAITING_TEST;
                $orderStatusRequiresInvField = Config::ORDER_STATUS_REQUIRES_INVESTIGATION_TEST;
                break;
            case Config::LIVE_ENVIRONMENT:
                $env = SwedBank::ENV_LIVE;
                $this->configuration->set(Config::TEST_ENVIRONMENT, 0);
                $this->configuration->set(Config::LIVE_ENVIRONMENT, 1);
                $tosField = Config::TERMS_AND_CONDITIONS_LIVE;
                $clientIdField = Config::CLIENT_ID_LIVE;
                $clientPasswordField = Config::CLIENT_PASSWORD_LIVE;
                $paymentMethodsField = Config::PAYMENT_METHODS_LIVE;
                $orderStatusSuccessField = Config::ORDER_STATUS_SUCCESS_LIVE;
                $debugModeField = Config::DEBUG_MODE_LIVE;
                $orderStatusAwaitingField = Config::ORDER_STATUS_AWAITING_LIVE;
                $orderStatusRequiresInvField = Config::ORDER_STATUS_REQUIRES_INVESTIGATION_LIVE;
                break;
        }

        $this->configuration->set($tosField, 1);
        $this->configuration->set($clientIdField, Tools::getValue($clientIdField));
        $this->configuration->set($clientPasswordField, Tools::getValue($clientPasswordField));
        $paymentMethods = [];

        //@todo we should probably separate enabled payment methods per environment
        if (Tools::getValue($paymentMethodsField.'_'.Config::PAYMENT_METHOD_CARD)) {
            $paymentMethods[] = Config::PAYMENT_METHOD_CARD;
        }

        if (Tools::getValue($paymentMethodsField.'_'.Config::PAYMENT_METHOD_SWEDBANK)) {
            $paymentMethods[] = Config::PAYMENT_METHOD_SWEDBANK;
        }

        if (Tools::getValue($paymentMethodsField.'_'.Config::PAYMENT_METHOD_SEB)) {
            $paymentMethods[] = Config::PAYMENT_METHOD_SEB;
        }

        if (Tools::getValue($paymentMethodsField.'_'.Config::PAYMENT_METHOD_DNB)) {
            $paymentMethods[] = Config::PAYMENT_METHOD_DNB;
        }

        if (Tools::getValue($paymentMethodsField.'_'.Config::PAYMENT_METHOD_NORDEA)) {
            $paymentMethods[] = Config::PAYMENT_METHOD_NORDEA;
        }

        if (Tools::getValue($paymentMethodsField.'_'.Config::PAYMENT_METHOD_DANSKE)) {
            $paymentMethods[] = Config::PAYMENT_METHOD_DANSKE;
        }

        if (Tools::getValue($paymentMethodsField.'_'.Config::PAYMENT_METHOD_PAYPAL)) {
            $paymentMethods[] = Config::PAYMENT_METHOD_PAYPAL;
        }

        if (Tools::getValue($paymentMethodsField.'_'.Config::PAYMENT_METHOD_CITADELE)) {
            $paymentMethods[] = Config::PAYMENT_METHOD_CITADELE;
        }

        if (!empty($paymentMethods)) {
            $this->configuration->set($paymentMethodsField, Tools::jsonEncode($paymentMethods));
        } elseif (empty($paymentMethods)) {
            $this->configuration->set($paymentMethodsField, '');
        }

        $this->configuration->set($orderStatusSuccessField, Tools::getValue($orderStatusSuccessField));
        $this->configuration->set($orderStatusAwaitingField, Tools::getValue($orderStatusAwaitingField));
        $this->configuration->set($orderStatusRequiresInvField, Tools::getValue($orderStatusRequiresInvField));
        $this->configuration->set($debugModeField, Tools::getValue($debugModeField));
        $languages = \Language::getLanguages(true);
        foreach ($languages as $lang) {
            if (Tools::getValue(Config::LANGUAGE_TYPE_BANKLINK.'_'.$lang['id_lang'])) {
                $this->saveLanguageObject(
                    $lang['id_lang'],
                    $this->context->shop->id,
                    Config::LANGUAGE_TYPE_BANKLINK,
                    $env,
                    Tools::getValue(Config::LANGUAGE_TYPE_BANKLINK.'_'.$lang['id_lang'])
                );
            }

            if (Tools::getValue(Config::LANGUAGE_TYPE_CARD.'_'.$lang['id_lang'])) {
                $this->saveLanguageObject(
                    $lang['id_lang'],
                    $this->context->shop->id,
                    Config::LANGUAGE_TYPE_CARD,
                    $env,
                    Tools::getValue(Config::LANGUAGE_TYPE_CARD.'_'.$lang['id_lang'])
                );
            }
        }

        Tools::redirectAdmin(
            $this->context->link->getAdminLink($this->controller_name).'&conf='.Config::CONFIRMATION_MESSAGE_ID
        );
    }

    /**
     * validates form before submit process
     * @return bool
     */
    protected function validateTestLiveSettings()
    {
        $environment = $this->getEnvironment();
        switch ($environment) {
            case Config::LIVE_ENVIRONMENT:
                if (!Tools::getValue(Config::LIVE_ENVIRONMENT)) {
                    $this->errors[] =
                        $this->module->l('Please enable live environment switch in order to proceed.', self::FILENAME);
                    return false;
                }
                $tosInputField = Config::TERMS_AND_CONDITIONS_LIVE;
                $clientIdField = Config::CLIENT_ID_LIVE;
                $clientPasswordField = Config::CLIENT_PASSWORD_LIVE;
                $paymentMethod = Config::PAYMENT_METHODS_LIVE;
                break;
            default:
            case Config::TEST_ENVIRONMENT:
                if (!Tools::getValue(Config::TEST_ENVIRONMENT)) {
                    $this->errors[] =
                        $this->module->l('Please enable test environment switch in order to proceed.', self::FILENAME);
                    return false;
                }
                $tosInputField = Config::TERMS_AND_CONDITIONS_TEST;
                $clientIdField = Config::CLIENT_ID_TEST;
                $clientPasswordField = Config::CLIENT_PASSWORD_TEST;
                $paymentMethod = Config::PAYMENT_METHODS_TEST;
                break;
        }

        if (!Tools::getValue($tosInputField.'_1')) {
            $this->errors[] =
                $this->module->l('You have to agree with terms and conditions in order to proceed.', self::FILENAME);
            return false;
        }

        $clientId = Tools::getValue($clientIdField);
        $clientPassword = Tools::getValue($clientPasswordField);

        if (!$clientId) {
            $this->errors[] = $this->module->l('Please fill client id field.', self::FILENAME);
        }

        if (!$clientPassword) {
            $this->errors[] = $this->module->l('Please fill password field.', self::FILENAME);
        }

        if (!empty($this->errors)) {
            return false;
        }

        if (!Validate::isCleanHtml($clientId)) {
            $this->errors[] = $this->module->l('invalid client id fields.');
        }

        if (Tools::getValue($paymentMethod.'_'.Config::PAYMENT_METHOD_CARD)) {
            $languages = \Language::getLanguages(true, $this->context->shop->id);
            foreach ($languages as $language) {
                if (!Tools::getValue(Config::LANGUAGE_TYPE_CARD.'_'.(int) $language['id_lang'])) {
                    $msg = $this->module
                        ->l('Missing page set id for language code %s. Payment method: Card payment', self::FILENAME);
                    $this->errors[] = sprintf($msg, $language['iso_code']);
                    continue;
                }

                if (!Validate::isInt(Tools::getValue(Config::LANGUAGE_TYPE_CARD.'_'.(int) $language['id_lang']))) {
                    $msg = $this->module
                        ->l('Incorrect page set id for language code %s. Payment method: Card payment', self::FILENAME);

                    $this->errors[] = sprintf($msg, $language['iso_code']);
                }
            }
        }

        if (!empty($this->errors)) {
            return false;
        }
        return true;
    }

    /**
     * @param string $environmentName - name for switch which is the only difference between live and test controllers
     * @param string $environmentLabelName - name of the label
     * @param bool $disableStatus - disables or enables for fields
     * @return array
     */
    private function getTermsAndConditionsFormFields($environmentName, $environmentLabelName, $disableStatus)
    {
        $this->context->smarty->assign(
            'termsAndConditionsLink',
            $this->module->getPathUri().'pdf/terms_and_conditions.pdf'
        );

        switch ($environmentName) {
            default:
            case Config::TEST_ENVIRONMENT:
                $termsAndConditionsInputName = Config::TERMS_AND_CONDITIONS_TEST;
                break;
            case Config::LIVE_ENVIRONMENT:
                $termsAndConditionsInputName = Config::TERMS_AND_CONDITIONS_LIVE;
                break;
        }
        $form = [];
        $form['form']['legend'] = array(
            'title' => $this->module->l('Terms and conditions', self::FILENAME),
            'icon' => 'icon-cogs'
        );

        $form['form']['input'][] = [
            'label' => '',
            'type' => 'hidden',
            'name' => 'paypal_payment_method_name',
        ];

        $form['form']['input'][] = [
            'label' => '',
            'type' => 'hidden',
            'name' => 'card_payment_method_name',
        ];

        $form['form']['input'][] = [
            'label' =>
                $this->context->smarty->fetch($this->getAdminTemplatePath().'terms-and-conditions-link.tpl'),
            'name' => $termsAndConditionsInputName,
            'type' => 'checkbox',
            'values' => array(
                'id' => 'id',
                'name' => 'name',
                'query' => array(
                    array(
                        'id' => 1,
                        'name' => $this->module->l('I agree and accept terms and conditions.', self::FILENAME)
                    )
                )
            )
        ];

        $form['form']['input'][] = [
            'label' => $environmentLabelName,
            'type' => 'switch',
            'class' => 't',
            'name' => $environmentName,
            'is_bool' => true,
            'values' => array(
                array(
                    'id' => $environmentName.'_on',
                    'value' => 1,
                    'label' => $this->module->l('Yes', self::FILENAME)
                ),
                array(
                    'id' => $environmentName.'_off',
                    'value' => 0,
                    'label' => $this->module->l('No', self::FILENAME)
                )
            )
        ];

        $form['form']['submit']['title'] = $this->module->l('Save', self::FILENAME);
        $form['form']['submit']['class'] = ($disableStatus) ?
            'btn btn-default pull-right disabled' :
            'btn btn-default pull-right';

        return $form;
    }

    /**
     * @param string $environment
     * @param bool $disableStatus - determines on page load if for fields should be enabled or disabled
     * @return array
     */
    private function getCountryFormFields($environment, $disableStatus)
    {
        switch ($environment) {
            default:
            case Config::TEST_ENVIRONMENT:
                $clientIdInputName = Config::CLIENT_ID_TEST;
                $clientPasswordInputName = Config::CLIENT_PASSWORD_TEST;
                $paymentMethodsInputName = Config::PAYMENT_METHODS_TEST;
                break;
            case Config::LIVE_ENVIRONMENT:
                $clientIdInputName = Config::CLIENT_ID_LIVE;
                $clientPasswordInputName = Config::CLIENT_PASSWORD_LIVE;
                $paymentMethodsInputName = Config::PAYMENT_METHODS_LIVE;
                break;
        }

        $countryBlockEnabled = $this->getStoredBankConfigurationName();
        if (!$countryBlockEnabled) {
            return [
                'form' => [
                    'input' => [
                        [
                            'type' => 'free',
                            'name' => 'no_country_block',
                            'label' => ''
                        ]
                    ]
                ]
            ];
        }

        $form = [];
        $form['form']['legend']['title'] = $this->getSelectedBankCountryName($countryBlockEnabled);
        $form['form']['input'][] = [
            'name' => $clientIdInputName,
            'label' => $this->module->l('Client id', self::FILENAME),
            'validate' => 'isCleanHtml',
            'col' => '3',
            'type' => 'text',
            'required' => true,
            'disabled' => $disableStatus
        ];
        $form['form']['input'][] = [
            'name' => $clientPasswordInputName,
            'label' => $this->module->l('Password', self::FILENAME),
            'validate' => 'isPasswd',
            'col' => '3',
            'type' => 'text',
            'required' => true,
            'disabled' => $disableStatus
        ];

        /** @var PaymentMethodProvider $paymentMethodProvider */
        $paymentMethodProvider = $this->module->get(PaymentMethodProvider::class);

        $countryLt = (bool) \Configuration::get(Config::BANK_LT);
        $countryLv = (bool) \Configuration::get(Config::BANK_LV);
        $countryEe = (bool) \Configuration::get(Config::BANK_EE);

        $activeCountry = $countryLt ? Config::BANK_LT :
            ($countryLv ? Config::BANK_LV :
                ($countryEe ? Config::BANK_EE : null));

        $paymentMethods = $paymentMethodProvider->getMethodsForCountry($activeCountry);


        $form['form']['input'][] = [
            'name' => $paymentMethodsInputName,
            'label' => $this->module->l('Payment methods', self::FILENAME),
            'type' => 'checkbox',
            'disabled' => $disableStatus,
            'values' => [
                'id' => 'id',
                'name' => 'name',
                'query' => $paymentMethods
            ],
        ];

        $form['form']['submit']['title'] = $this->module->l('Save', self::FILENAME);
        $form['form']['submit']['class'] = ($disableStatus) ?
            'btn btn-default pull-right disabled' :
            'btn btn-default pull-right';

        $languages = \Language::getLanguages(true, $this->context->shop->id);

        if (empty($languages)) {
            $this->errors[] = $this->module->l('No languages found for configuration.', self::FILENAME);
            return $form;
        }

        $formGroupClass = 'hide';
        $form['form']['input'][] = [
            'type' => 'free',
            'form_group_class' => 'banklink_text '.$formGroupClass,
            'label' => '',
            'name' => 'banklink_text',
        ];

        foreach ($languages as $language) {
            $form['form']['input'][] = $this->formatPaymentLanguageField(
                'banklink_'.$language['id_lang'],
                $language['iso_code'],
                'select',
                $formGroupClass,
                $disableStatus
            );
        }

        $form['form']['input'][] = [
            'type' => 'free',
            'form_group_class' => Config::LANGUAGE_TYPE_CARD.'_text '.$formGroupClass,
            'label' => '',
            'name' => Config::LANGUAGE_TYPE_CARD.'_text',
        ];

        foreach ($languages as $language) {
            $form['form']['input'][] = $this->formatPaymentLanguageField(
                Config::LANGUAGE_TYPE_CARD.'_'.$language['id_lang'],
                $language['iso_code'],
                'text',
                $formGroupClass,
                $disableStatus
            );
        }

        // used for recognising on javascript level
        if ($disableStatus) {
            $form['form']['input'][] = [
                'name' => 'disable_form',
                'type' => 'hidden',
                'label' => '',
            ];
        }
        return $form;
    }

    /**
     * @param string $environment
     * @param $disableStatus - determines on page load if for fields should be enabled or disabled
     * @return array
     */
    private function getOrderStatusFormFields($environment, $disableStatus)
    {
        switch ($environment) {
            default:
            case Config::TEST_ENVIRONMENT:
                $orderStatusSuccessInputName = Config::ORDER_STATUS_SUCCESS_TEST;
                $orderStatusAwaitingInputName = Config::ORDER_STATUS_AWAITING_TEST;
                $orderStatusRequiresInvName = Config::ORDER_STATUS_REQUIRES_INVESTIGATION_TEST;
                $debugModeInputName = Config::DEBUG_MODE_TEST;
                break;
            case Config::LIVE_ENVIRONMENT:
                $orderStatusSuccessInputName = Config::ORDER_STATUS_SUCCESS_LIVE;
                $orderStatusAwaitingInputName = Config::ORDER_STATUS_AWAITING_LIVE;
                $orderStatusRequiresInvName = Config::ORDER_STATUS_REQUIRES_INVESTIGATION_LIVE;
                $debugModeInputName = Config::DEBUG_MODE_LIVE;
                break;
        }

        $debugFile = $this->context->link->getAdminLink('AdminSwedBankLogFileDisplay');

        $this->context->smarty->assign('debugFileUrl', $debugFile);

        $debugFileHtml = $this->context->smarty->fetch(
            $this->module->getLocalPath().'views/templates/admin/log-file.tpl'
        );

        $allStates = OrderState::getOrderStates($this->context->language->id);
        $paidFlagStates = $this->getPaidFlagStates($allStates);
        $excludePaidStates = $this->getNonPaidFlagStates($allStates);

        $data = array();
        $data['form']['legend']['title'] = $this->module->l('Additional settings', self::FILENAME);
        $data['form']['input'][] =
            array(
                'label' => $this->module->l('Order status - successful payment', self::FILENAME),
                'name' => $orderStatusSuccessInputName,
                'type' => 'select',
                'disabled' => $disableStatus,
                'options' => array(
                    'query' => $paidFlagStates,
                    'id' => 'id_order_state',
                    'name' => 'name',
                ),
            );
        $data['form']['input'][] =
            array(
                'label' => $this->module->l('Order status - awaiting payment', self::FILENAME),
                'name' => $orderStatusAwaitingInputName,
                'type' => 'select',
                'disabled' => $disableStatus,
                'options' => array(
                    'query' => $excludePaidStates,
                    'id' => 'id_order_state',
                    'name' => 'name',
                ),
            );
        $data['form']['input'][] =
            array(
                'label' => $this->module->l('Order status - requires investigation', self::FILENAME),
                'name' => $orderStatusRequiresInvName,
                'type' => 'select',
                'disabled' => $disableStatus,
                'options' => array(
                    'query' => $excludePaidStates,
                    'id' => 'id_order_state',
                    'name' => 'name',
                ),
            );
        $data['form']['input'][] =
            array(
                'label' => $this->module->l('Enable debug mode', self::FILENAME),
                'type' => 'switch',
                'name' => $debugModeInputName,
                'class' => 't',
                'is_bool' => true,
                'disabled' => $disableStatus,
                'values' => array(
                    array(
                        'id' => 'activate_on',
                        'value' => 1,
                        'label' => $this->module->l('Yes', self::FILENAME)
                    ),
                    array(
                        'id' => 'activate_off',
                        'value' => 0,
                        'label' => $this->module->l('No', self::FILENAME)
                    )
                ),
                'desc' => $debugFileHtml,
            );

        $data['form']['submit']['title'] =  $this->module->l('Save', self::FILENAME);
        $data['form']['submit']['class'] = ($disableStatus) ?
            'btn btn-default pull-right disabled' :
            'btn btn-default pull-right';
        return $data;
    }

    /**
     * Render top navigation in module controllers
     *
     * @return string
     */
    private function renderNavigationTop()
    {
        $tabsCollection = new Collection('Tab', $this->context->language->id);
        $tabsCollection->where('module', '=', $this->module->name);
        $tabsCollection->where('id_parent', '=', '-1');
        $tabsCollection->where('class_name', '!=', SwedBank::ADMIN_LOG_FILE_DISPLAY_CONTROLLER);
        
        $tabs = $this->buildTabs($tabsCollection);

        $this->context->smarty->assign(array(
            'menuTabs' => $tabs,
        ));

        return $this->context->smarty->fetch(
            $this->getAdminTemplatePath().'navigation.tpl'
        );
    }
    
    private function buildTabs(Collection $tabs)
    {
        $menuTabs = array();
        
        /** @var Tab $tab */
        foreach ($tabs as $tab) {
            $menuTab = array(
                'name' => $tab->name,
                'href' => $this->context->link->getAdminLink($tab->class_name),
                'current' => $this->isActiveTab($tab),
                'className' => $tab->class_name,
            );

            $menuTabs[] = $menuTab;
        }
        
        return $menuTabs;
    }
    
    private function isActiveTab(Tab $tab)
    {
        if ($tab->class_name == $this->controller_name) {
            return true;
        }
        
        if ($idChildTab = Tab::getIdFromClassName($this->controller_name)) {
            $child = new Tab($idChildTab);
            
            // If $tab is a parent of current active tab - it's active as well
            if ($child->id_parent == $tab->id) {
                return true;
            }
        }
        
        return false;
    }

    private function renderNavigationBottom()
    {
        return $this->context->smarty->fetch(
            $this->getAdminTemplatePath().'navigation-bottom.tpl'
        );
    }

    /**
     * gets translatable country name according to selected configuration
     * @param $selectedConfigurationName
     * @return string
     */
    private function getSelectedBankCountryName($selectedConfigurationName)
    {
        switch ($selectedConfigurationName) {
            case Config::BANK_LT:
                return $this->module->l('Lithuania', self::FILENAME);
            case Config::BANK_LV:
                return $this->module->l('Latvia', self::FILENAME);
            case Config::BANK_EE:
                return $this->module->l('Estonia', self::FILENAME);
        }
        return '';
    }

    /**
     * Get environment based on context controller
     *
     * @return string
     */
    protected function getEnvironment()
    {
        switch ($this->controller_name) {
            default:
            case SwedBank::ADMIN_TEST_GENERAL_SETTINGS_CONTROLLER:
                $env = Config::TEST_ENVIRONMENT;
                break;
            case SwedBank::ADMIN_LIVE_GENERAL_SETTINGS_CONTROLLER:
                $env = Config::LIVE_ENVIRONMENT;
                break;
        }

        return $env;
    }

    private function formatPaymentLanguageField($name, $isoCode, $type, $class, $disableStatus)
    {
        switch ($type) {
            case 'text':
                return [
                    'name' => $name,
                    'type' => $type,
                    'form_group_class' => $class,
                    'class' => '',
                    'label' => $isoCode,
                    'desc' => '',
                    'col' => '2',
                    'required' => true,
                    'disabled' => $disableStatus,
                    'validate' => 'isCleanHtml'
                ];
            default:
                $bankLtEnabled = (bool) \Configuration::get('SB_BANK_LT');
                $bankLvEnabled = (bool) \Configuration::get('SB_BANK_LV');
                $bankEeEnabled = (bool) \Configuration::get('SB_BANK_EE');

                if ($bankEeEnabled) {
                    $languages = Config::getFieldsFormLanguages('EE');
                } elseif ($bankLtEnabled) {
                    $languages = Config::getFieldsFormLanguages('LT');
                } elseif ($bankLvEnabled) {
                    $languages = Config::getFieldsFormLanguages('LV');
                } else {
                    $languages = Config::getFieldsFormLanguages();
                }

                return [
                    'name' => $name,
                    'type' => $type,
                    'label' => $isoCode,
                    'desc' => '',
                    'disabled' => $disableStatus,
                    'form_group_class' => $class,
                    'class' => '',
                    'required' => true,
                    'options' => [
                        'query' => $languages,
                        'id' => 'id',
                        'name' => 'name'
                    ]
                ];
        }
    }

    private function refillFieldValues()
    {
        $environment = $this->getEnvironment();
        switch ($environment) {
            default:
            case Config::TEST_ENVIRONMENT:
                $tosField = Config::TERMS_AND_CONDITIONS_TEST;
                $envField = Config::TEST_ENVIRONMENT;
                $clientIdField = Config::CLIENT_ID_TEST;
                $clientPasswordField = Config::CLIENT_PASSWORD_TEST;
                $paymentMethodsField = Config::PAYMENT_METHODS_TEST;
                $orderStatusSuccessField = Config::ORDER_STATUS_SUCCESS_TEST;
                $debugModeField = Config::DEBUG_MODE_TEST;
                $orderStatusAwaitingField = Config::ORDER_STATUS_AWAITING_TEST;
                break;
            case Config::LIVE_ENVIRONMENT:
                $tosField = Config::TERMS_AND_CONDITIONS_LIVE;
                $envField = Config::LIVE_ENVIRONMENT;
                $clientIdField = Config::CLIENT_ID_LIVE;
                $clientPasswordField = Config::CLIENT_PASSWORD_LIVE;
                $paymentMethodsField = Config::PAYMENT_METHODS_LIVE;
                $orderStatusSuccessField = Config::ORDER_STATUS_SUCCESS_LIVE;
                $debugModeField = Config::DEBUG_MODE_LIVE;
                $orderStatusAwaitingField = Config::ORDER_STATUS_AWAITING_LIVE;
                break;
        }
        $this->fields_value[$tosField.'_1'] = 1;
        $this->fields_value[$envField] = Tools::getValue($envField);
        $this->fields_value[$clientIdField] = Tools::getValue($clientIdField);
        $this->fields_value[$clientPasswordField] = Tools::getValue($clientPasswordField);
        $paymentMethodIds = Config::getPaymentMethodIds();
        foreach ($paymentMethodIds as $id) {
            if (Tools::getValue($paymentMethodsField.'_'.$id)) {
                $this->fields_value[$paymentMethodsField.'_'.$id] = 1;
            }
        }
        $this->fields_value[$orderStatusSuccessField] = Tools::getValue($orderStatusSuccessField);
        $this->fields_value[$orderStatusAwaitingField] = Tools::getValue($orderStatusAwaitingField);
        $this->fields_value[$debugModeField] = Tools::getValue($debugModeField);

        $languages = \Language::getLanguages(true);

        foreach ($languages as $language) {
            $this->fields_value[Config::LANGUAGE_TYPE_BANKLINK.'_'.$language['id_lang']] =
                Tools::getValue(Config::LANGUAGE_TYPE_BANKLINK.'_'.$language['id_lang']);
            $this->fields_value[Config::LANGUAGE_TYPE_CARD.'_'.$language['id_lang']] =
                Tools::getValue(Config::LANGUAGE_TYPE_CARD.'_'.$language['id_lang']);
        }
    }

    private function saveLanguageObject($idLang, $idShop, $type, $environment, $selectedLanguage)
    {
        $idPaymentLanguage = \SwedbankLanguage::getId($idLang, $idShop, $type, $environment);
        $swedbankLanguage = new \SwedbankLanguage($idPaymentLanguage);
        $swedbankLanguage->id_language = $idLang;
        $swedbankLanguage->id_shop = $idShop;
        $swedbankLanguage->type = $type;
        $swedbankLanguage->selected_language = $selectedLanguage;
        $swedbankLanguage->environment = $environment;
        $swedbankLanguage->save();
    }

    private function isCheckboxMarked($specificCheckboxId, $bankLinkCheckboxes = false)
    {
        $environment = $this->getEnvironment();

        switch ($environment) {
            case Config::TEST_ENVIRONMENT:
                $paymentMethodsField = Config::PAYMENT_METHODS_TEST;
                break;
            default:
                $paymentMethodsField = Config::PAYMENT_METHODS_LIVE;
                break;
        }

        $paymentMethods = (array) Tools::jsonDecode($this->configuration->get($paymentMethodsField));
        if ($specificCheckboxId) {
            return in_array($specificCheckboxId, $paymentMethods);
        }

        if ($bankLinkCheckboxes) {
            $bankLinkIds = Config::getBankLinkPaymentMethodIds();
            return !empty(array_intersect($bankLinkIds, $paymentMethods));
        }
        return false;
    }

    private function getPaidFlagStates(array $allStates)
    {
        $result = [];
        foreach ($allStates as $state) {
            if ($state['paid']) {
                $result[] = $state;
            }
        }

        return $result;
    }

    private function getNonPaidFlagStates(array $allStates)
    {
        $result = [];
        foreach ($allStates as $state) {
            if (!$state['paid']) {
                $result[] = $state;
            }
        }

        return $result;
    }
}
